/*
 * Convert a PostScript file on stdin to an LZ-compressed version
 * of the same file, prefixed by the appropriate "/LZWDecode filter"
 * command so it can be handled by a *Level 2* PostScript interpreter.
 */

#include <stdio.h>

/*
 * Various constants
 */
#define EOD		257	/* End of Data marker */
#define CLR		256	/* Clear Table Marker */
#define MAXSTRINGS	4096	/* using 12 bit max codes */

/*
 * This is the structure for a single entry into the string table
 */
typedef struct {
	short prefix;
	short suffix;
	short next;
} stringrec;

int codeWidth = 9;
int bitsInAcc = 0;		/* Used by code send function */
unsigned long acc;		/* =bits left over from a sendcode */
int nextIndex;			/* Used by newentry function */
stringrec stringarray[4096];	/* This is the string table */

void Initialize(void);
void SendCode(short theCode);
int FindString(short *prefix, short suffix);
int main(int argc, char *argv[]);
void AddString(short prefix, short suffix);

void 
Initialize()
{
	int     index;

	for (index = 0; index < 256; index++) {
		stringarray[index].prefix = -1;
		stringarray[index].suffix = (char)index;
		stringarray[index].next = -1;
	}
	nextIndex = EOD + 1;
	codeWidth = 9;
}

void 
SendCode(short theCode)
{
	acc += (long) theCode << (32 - codeWidth - bitsInAcc);
	bitsInAcc += codeWidth;
	while (bitsInAcc >= 8) {
		fputc(acc >> 24, stdout);
		acc = acc << 8;
		bitsInAcc -= 8;
	}
	if (theCode == CLR) {
		Initialize();
	}
	if (theCode == EOD && bitsInAcc != 0) {
		fputc(acc >> 24, stdout);
	}
}

int 
main(int argc, char *argv[])
{
	short   lastCode;
	short   thisCode;
	double  comppct;

	if (argc != 1) {
		fprintf(stderr, "lzps - Too many arguments.\n");
		fflush(stderr);
		exit (1);
	}
	if ((lastCode = fgetc(stdin)) == EOF) {
		fprintf(stderr, "lzps - Empty file.\n");
		exit(1);
	}
	fprintf(stdout, "%%!PS-\ncurrentfile /LZWDecode filter cvx exec\n");

	SendCode(CLR);

	while ((thisCode = fgetc(stdin)) != EOF) {
		if (FindString(&lastCode, thisCode) == 0) {
			SendCode(lastCode);
			AddString(lastCode, thisCode);
			lastCode = thisCode;
		}
	}
	SendCode(lastCode);
	SendCode(EOD);

	exit(0);
}

int 
FindString(short *prefix, short suffix)
{
	short   index = *prefix;

	while (index != -1) {
		if (stringarray[index].prefix == *prefix &&
		    stringarray[index].suffix == suffix) {
			*prefix = index;
			return (1);
		} else {
			index = stringarray[index].next;
		}
	}
	return (0);
}

void 
AddString(short prefix, short suffix)
{

	stringarray[nextIndex].prefix = prefix;
	stringarray[nextIndex].suffix = suffix;
	stringarray[nextIndex].next = stringarray[prefix].next;
	stringarray[prefix].next = nextIndex;

	if (++nextIndex >> codeWidth) {
		if (++codeWidth > 12) {
			--codeWidth;
			SendCode(CLR);
		}
	}
}
