/*
 * Copyright (c) 1992 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @(#) $Header: psfile.h,v 1.4 93/10/11 00:15:07 van Exp $ (LBL)
 */

#ifndef wb_psfile_h
#define wb_psfile_h

#include <sys/param.h>

/*
 * A class for breaking up a PostScript file into preamble and pages.
 * The file must be DSC conformant (see Appendix G of The PosctScript
 * Language Reference Manual, 2nd Ed.), in which case PFfile::valid()
 * is true.  Otherwise, it is false and the behavior of all other methods
 * is undefined.
 */
class PSfile {
    private:
	struct pageinfo {
		off_t offset;
		int length;
	};
    public:
	PSfile(int fd);
	~PSfile();
	inline int valid() const { return npage >= 0; }
	inline int npages() const { return npage; }
	inline int preamble_length() const { return info[0].length; }
	int page_length(int pageno);
	const char* page(int pageno, int& len);
	inline const char* preamble(int& len) { return (page(0, len)); }
	void release(const char*, int len);
    private:
	int dostat();
	int scanhdr(pageinfo&);
	void scaninfo(int pageno);
	int fsearch(const char* cp, int len, const char* target) const;
	int bsearch(const char* cp, int len, const char* target) const;
	int match(const char* nt, const char* s, int len) const;
	off_t nextpage(off_t prev);

	int fd;
	int filesize;
	int lastnum;
	int npage;		/* number of pages */
	pageinfo* info;
};

#endif
