# Copyright (c) 1993 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: wbi_subr.tcl,v 1.12 93/10/16 01:01:38 van Exp $ (LBL)

option add WbImport.xid 	0			startupFile
option add WbImport.tmpfile	/tmp/wbImport.ps	startupFile
option add WbImport.scanCommand	doWbScan		startupFile
option add WbImport.xwdCommand	doWbXwd			startupFile
option add WbImport.propName	PSFILE			startupFile

set maxwbpage 0
set wb_request 0
set wborientation 0
set do_pagescan 1

proc fileInPath {f} {
    global env
    if {$f == "" || $f == "-"} {
	return 0
    }
    if {[string index $f 0] == "/"} {
	return [file executable $f]
    }
    foreach d [split $env(PATH) :] {
	if {[file executable $d/$f]} {
	    return 1
	}
    }
    return 0
}

proc wbi_request {cmd} {
    global wbwindow prop_name wb_request
    if {$wb_request == 0} {
	set wb_request 1
	setProperty $wbwindow $prop_name $cmd
    }
}

proc wbi_request_async {cmd} {
    global wbwindow prop_name wb_request
    if {$wb_request == 0} {
	setProperty $wbwindow $prop_name $cmd
    }
}

proc wbi_scan {} {
    global wbi_label tmpfile scan_command wborientation do_pagescan
    $wbi_label configure -text "Scanning..."
    update idletasks
    exec $scan_command -$wborientation > $tmpfile
    set do_pagescan 0
    wbi_request_async "$tmpfile"
    $wbi_label configure -text "Ready"
    set do_pagescan -1
}

proc wbi_xwd {} {
    global wbi_label tmpfile xwd_command wborientation do_pagescan
    $wbi_label configure -text "click on a window..."
    update idletasks
    exec $xwd_command -$wborientation > $tmpfile
    set do_pagescan 0
    wbi_request_async "$tmpfile"
    $wbi_label configure -text "Ready"
    set do_pagescan -1
}

proc wbi_file {n} {
    global wbifilelist wbifileslot
    if {$wbifileslot($n)} {
	wbi_request_async "#$wbifileslot($n)"
    } else {
	set cmd "[winfo id .] $n [.b.lb get $n] $wbifilelist($n)"
	wbi_request $cmd
    }
}

proc getNextPageName {n} {
    set cmd "[winfo id .] [expr 65537+$n] =$n"
    wbi_request $cmd
}

proc handleClientEvent {n slot} {
    global wb_request wbifilelist wbifileslot maxwbpage prop_name
    set wb_request 0
    if {$n == 65536} {
	global wborientation
	set mp [expr $slot&0xffff]
	set wborientation [expr $slot>>16]
	if {$mp > $maxwbpage} {
	    getNextPageName [expr $maxwbpage+1]
	    set maxwbpage $mp
	}
	return
    }
    if {$n >= 65537} {
	set n [expr $n-65537]
	if {$slot>0} {
	    set t [getProperty [winfo id .] $prop_name del]
	    set l [addItemToFileList "$t ($slot)" /dev/null]
	    set wbifileslot($l) $slot
	}
	if {$n < $maxwbpage} {
	    getNextPageName [expr $n+1]
	}
	return
    }
    if {$slot} {
	if {$slot > $maxwbpage} {
	    set maxwbpage $slot
	}
	set wbifileslot($n) $slot
	set t [.b.lb get $n]
	.b.lb delete $n
	.b.lb insert $n "$t ($slot)"
	.b.lb select from $n
    } else {
	puts stderr "-import of $wbifilelist($n) failed"
    }
}

proc wbi_quit {} {
    global tmpfile
    exec rm -f $tmpfile
    destroy .
}

proc fullPathName {f} {
    if {[file readable $f] == 0} {
	puts stderr "-wbimport: can't find file `$f'"
    }
    set d [file dirname $f]
    case $d in {
	.	{ set d [pwd] }
	./*	{ set d [pwd]/$d }
	..	{ set d [file dirname [pwd]] }
	../*	{ set d [pwd]/$d }
	/*	{ }
	default	{ set d [pwd]/$d }
    }
    return $d/[file tail $f]
}

proc addItemToFileList {iname ifile} {
    global wbifilelist wbifileslot
    set i [.b.lb size]
    .b.lb insert end $iname
    set wbifilelist($i) [fullPathName $ifile]
    set wbifileslot($i) 0
    return $i
}

proc makeFileList {fname} {
    set f [open $fname]
    for {set i 0} {[gets $f line]>0} {incr i 1} {
	if {[llength $line]>1} {
	    addItemToFileList [lindex $line 0] [lindex $line 1]
	} else {
	    if {[regexp -indices -nocase .ps\$ $line m]} {
		set n [string range $line 0 [expr [lindex $m 0]-1]]
	    } else {
		set n $line
	    }
	    addItemToFileList $n $line
	}
    }
    close $f
}

# ask the server what page it's on & if it's higher than what we
# expect, query it for page names.
proc addCurrentPages {} {
    global do_pagescan
    if {$do_pagescan>0} {
	set cmd "[winfo id .] 65536 ?"
	wbi_request $cmd
    } else {
	if {$do_pagescan<0} {
	    set do_pagescan 1
	}
    }
    after 2000 addCurrentPages
}

proc makeFileListBox {} {
    global wbifilelist wbifileslot wbi_label
    frame .b
    scrollbar .b.lbscroll -relief sunken -command ".b.lb yview"
    listbox .b.lb -yscroll ".b.lbscroll set" -relief sunken -setgrid 1 \
		-geometry 6x10
    pack append .b \
	.b.lbscroll {right filly} \
	.b.lb {left expand fill}
    tk_listboxSingleSelect .b.lb
    bind .b.lb <ButtonRelease-1> \
	    "wbi_file \[.b.lb curselection\]; adjustScroll"

    pack append . .b {bottom expand fill}

    pack after $wbi_label \
	[button .f.next -text "Next" -command "nextFile"] {left padx 5 pady 5}
}

proc adjustScroll {} {
    set sb [.b.lbscroll get]
    if {[lindex $sb 0] > [lindex $sb 1]} {
	# box too big for window - we might need to adjust scrollbar
	# to keep selected item in the middle.
	set m [expr ([lindex $sb 1]+1)/2]
	if {$m > 1} {
	    incr m -1
	}
	set c [.b.lb curselection]
	if {$c >= [expr [lindex $sb 0]-$m]} {
	    .b.lb yview [expr [lindex $sb 0]-[lindex $sb 1]+1]
	} else {
	    if {$c >= $m} {
		.b.lb yview [expr $c-$m]
	    }
	}
    }
}

proc nextFile {} {
    set c [.b.lb curselection]
    if {$c == ""} {
	set c -1
    }
    incr c 1
    if {$c < [.b.lb size]} {
	.b.lb select from $c
	wbi_file $c
	adjustScroll
    }
}

proc makeWin {} {
    global wbi_label scan_command xwd_command
    frame .f
    pack append . .f {top fill}
    set wbi_label .f.label
    pack append .f \
	[label $wbi_label -text "Ready"] {bottom fillx}
    if {[fileInPath $scan_command]} {
	pack append .f \
	    [button .f.scan -text "Scan" -command "wbi_scan"] \
	    {left padx 5 pady 5}
    }
    if {[fileInPath $xwd_command]} {
	pack append .f \
	    [button .f.xwd -text "Xwin" -command "wbi_xwd"] \
	    {left padx 5 pady 5}
    }
    pack append .f \
	[button .f.quit -text "Quit" -command "wbi_quit"] {left padx 5 pady 5}

    makeFileListBox
}

proc winChoice {wlist {w .wd}} {
    catch {destroy $w}
    toplevel $w
    frame $w.top
    frame $w.bot
    pack append $w $w.top {top fill expand} $w.bot {top fill expand}
    pack append $w.top \
	[message $w.top.msg -justify center -text "Which wb?"] \
	{top fill expand}
    
    foreach i $wlist {
	button $w.bot.$i -anchor w -text "   $i ([getWindowName $i])" \
	    -command "global wbwindow; set wbwindow $i; destroy $w"
	pack append $w.bot $w.bot.$i {top fill expand padx 4 pady 8}
    }
    button $w.bot.cancel -text "Quit" \
	-command "exit 1"
    pack append $w.bot $w.bot.cancel {top fill expand}

    grab set $w
    tkwait window $w
}

set wbwindow [option get . xid WbImport]
if { $wbwindow == 0 } {
    set wbids [getWindowByClass wb]
    if {[llength $wbids] != 1} {
	if {[llength $wbids] == 0} {
	    puts stderr "-wbimport: can't find an active `wb'"
	    exit 1
	}
	winChoice $wbids
    } else {
	set wbwindow [lindex $wbids 0]
    }
}
set wbwinname [getWindowName $wbwindow]
if {$wbwinname != ""} {
    wm title . "wbi: $wbwinname"
}
set tmpfile [option get . tmpfile WbImport]
set scan_command [option get . scanCommand WbImport]
set xwd_command [option get . xwdCommand WbImport]
set prop_name [option get . propName WbImport]

makeWin

proc wbi_run {f} {
    if {$f != "."} {
	makeFileList $f
    }
    addCurrentPages
    tkwait window .
}
