/*
 * Copyright (c) 1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: wbimport.c,v 1.6 93/10/16 01:01:40 van Exp $ (LBL)";
#endif

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <tkInt.h>

Tcl_Interp* tcl;
Tk_Window tkWin;
static char* display;

extern char wbi_init[];
extern char TK_Init[];

Tk_ArgvInfo disparg[] = {
    { "-display", TK_ARGV_STRING, NULL, (char*)&display, "X display name"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}
};

Tk_ArgvInfo allarg[] = {
    { "-display", TK_ARGV_STRING, NULL, (char*)&display, "X display name"},
    { "-id", TK_ARGV_OPTION_VALUE, NULL, "WbImport.xid", "wb window id"},
    { "-tmp", TK_ARGV_OPTION_VALUE, NULL, "WbImport.tmpfile", "tmp filename"},
    { "-scan", TK_ARGV_OPTION_VALUE, NULL, "WbImport.scanCommand", "scan command to exec"},
    { "-xwd", TK_ARGV_OPTION_VALUE, NULL, "WbImport.xwdCommand", "X window dump command to exec"},
    { "-prop", TK_ARGV_OPTION_VALUE, NULL, "WbImport.propName", "Import X property name"},
    {NULL, TK_ARGV_END, NULL, NULL, NULL}
};

static u_long last_atom;

int SetWindowProperty(ClientData clientData, Tcl_Interp *tcl, int argc,
		      char *argv[])
{
	int stat = TCL_ERROR;

	if (argc != 4) {
		Tcl_AppendResult(tcl, "wrong # args: should be \"", argv[0],
				 " xwinid propname value\"", NULL);
	} else {
		Display* xd = Tk_Display(tkWin);
		u_long w = strtol(argv[1], NULL, 0);
		if (w == 0)
			w = Tk_WindowId(tkWin);
		last_atom = XInternAtom(xd, argv[2], False);
		XChangeProperty(xd, w, last_atom, XA_STRING, 8, PropModeReplace,
				(u_char*)argv[3], strlen(argv[3]));
		stat = TCL_OK;
	}
	return (stat);
}

int GetWindowProperty(ClientData clientData, Tcl_Interp *tcl, int argc,
		      char *argv[])
{
	int del = False;
	Display* xd = Tk_Display(tkWin);
	u_long patom = XInternAtom(xd, argv[2], False);
	u_long w = strtol(argv[1], NULL, 0);
	Atom ptype;
	int pfmt;
	u_long pitems, bytes_left;
	u_char* prop = 0;

	if (argc != 3) {
		if (argc != 4) {
			Tcl_AppendResult(tcl, "wrong # args: should be \"",
				argv[0], " xwinid propname [del]\"", NULL);
			return (TCL_ERROR);
		}
		del = True;
	}
	if (w == 0)
		w = Tk_WindowId(tkWin);
	XGetWindowProperty(xd, w, patom, 0, 1024, del, XA_STRING, &ptype,
			&pfmt, &pitems, &bytes_left, &prop);
	Tcl_AppendResult(tcl, prop, NULL);
	XFree((char*)prop);
	return (TCL_OK);
}

int LookForClientEvents(ClientData clientData, XEvent* ev)
{
	char resp[64];

	if (ev->xclient.type != ClientMessage ||
	    ev->xclient.message_type != last_atom ||
	    ev->xclient.format != 32)
		return (0);

	sprintf(resp, "handleClientEvent %lu %lu", ev->xclient.data.l[0],
		ev->xclient.data.l[1]);
	Tcl_VarEval(tcl, resp, NULL);
	return (1);
}

int GetWindowByClass(ClientData clientData, Tcl_Interp *tcl, int argc,
		      char *argv[])
{
	Display* xd = Tk_Display(tkWin);
	Window w = (Window)clientData;
	Window root, parent;
	Window* child;
	u_int nc;
	int i;

	if (argc != 2) {
		Tcl_AppendResult(tcl, "wrong # args: should be \"",
				argv[0], " classname\"", NULL);
		return (TCL_ERROR);
	}
	if (w == 0)
		w = RootWindowOfScreen(Tk_Screen(tkWin));
	if (XQueryTree(xd, w, &root, &parent, &child, &nc) == 0) {
		Tcl_AppendResult(tcl, argv[0], " cant get root window tree",
				NULL);
		return (TCL_ERROR);
	}
	for (i = (int)nc; --i >= 0; ) {
		XClassHint class;
		if (XGetClassHint(xd, child[i], &class)) {
			if (class.res_class) {
				if (strcmp(class.res_class, argv[1]) == 0 &&
				    class.res_name &&
				    class.res_name[0] != '@') {
					char wid[32];
					sprintf(wid, "0x%lx", child[i]);
					Tcl_AppendElement(tcl, wid, 0);
				}
				XFree(class.res_class);
			}
			if (class.res_name)
				XFree(class.res_name);
		}
		GetWindowByClass((ClientData)child[i], tcl, argc, argv);
	}
	if (child)
		XFree((char*)child);
	return (TCL_OK);
}

int GetWindowName(ClientData clientData, Tcl_Interp *tcl, int argc,
		  char *argv[])
{
	Display* xd = Tk_Display(tkWin);
	u_long w;
	char* name;

	if (argc != 2) {
		Tcl_AppendResult(tcl, "wrong # args: should be \"",
				argv[0], " xwinid\"", NULL);
		return (TCL_ERROR);
	}
	w = strtol(argv[1], NULL, 0);
	if (w == 0) {
		Tcl_AppendResult(tcl, argv[0], ": invalid window id \"",
				argv[1], "\"", NULL);
		return (TCL_ERROR);
	}
	if (XFetchName(xd, w, &name) != 0) {
		Tcl_AppendResult(tcl, name, NULL);
		XFree((char*)name);
	}
	return (TCL_OK);
}

int main(int argc, char* argv[])
{
	tcl = Tcl_CreateInterp();
	(void) Tk_ParseArgv(tcl, NULL, &argc, argv, disparg,
			    TK_ARGV_NO_DEFAULTS);
	tkWin = Tk_CreateMainWindow(tcl, display, "wbimport");
	if (tkWin == NULL) {
		fprintf(stderr, "%s: %s\n", argv[0], tcl->result);
		exit(1);
	}
	Tk_SetClass(tkWin, "WbImport");
	if (Tk_ParseArgv(tcl, tkWin, &argc, argv, allarg, 0) != TCL_OK) {
		fprintf(stderr, "%s: %s\n", argv[0], tcl->result);
		exit(1);
	}

	Tcl_CreateCommand(tcl, "setProperty", SetWindowProperty, 0, 0);
	Tcl_CreateCommand(tcl, "getProperty", GetWindowProperty, 0, 0);
	Tcl_CreateCommand(tcl, "getWindowByClass", GetWindowByClass, 0, 0);
	Tcl_CreateCommand(tcl, "getWindowName", GetWindowName, 0, 0);
	Tk_CreateGenericHandler(LookForClientEvents, 0);

	if (Tcl_VarEval(tcl, TK_Init, wbi_init, NULL) != TCL_OK) {
		fprintf(stderr, "%s: %s\n", argv[0], tcl->result);
		exit(1);
	}
	Tk_MapWindow(tkWin);
	if (Tcl_VarEval(tcl, "wbi_run ", argc>1? argv[1]:".", NULL) != TCL_OK) {
		fprintf(stderr, "%s: %s\n", argv[0], tcl->result);
		exit(1);
	}
	exit(0);
}
